#include <fstream>
#include <iomanip>
#include <string>
#include <iostream>
#include "channelflow/flowfield.h"
#include "channelflow/diffops.h"
#include "channelflow/symmetry.h"
#include "channelflow/utilfuncs.h"

using namespace std;
using namespace channelflow;

int main(int argc, char* argv[]) {

  string purpose("apply a differential operation to a given field");

  ArgList args(argc, argv, purpose);

  const string op     = args.getstr(3, "<operation>", "ddx|ddz|grad|lapl|curl|div");
  const string iname  = args.getstr(2, "<flowfield>", "input field");
  const string oname  = args.getstr(1, "<flowfield>", "output field");
  args.check();

  FlowField u(iname);
  u.makeSpectral();

  FlowField v;

  // Note: could also use v = grad(u) etc, but grad(u,v) is more efficient

  if (op == "ddx")
    xdiff(u,v);

  else if (op == "ddz")
    zdiff(u,v);

  else if (op == "grad")
    grad(u,v);

  else if (op == "lapl")
    lapl(u,v);

  else if (op == "curl")
    curl(u,v);

  else if (op == "div")
    div(u,v);

  else {
    cerr << "error: unrecognized operation \"" << op << "\"" << endl;
    exit(1);
  }

  v.save(oname);
}
